@@ -503,20 +503,20 @@ pcap_set_rfmon(pcap_t *a, int b)
 int
 pcap_set_timeout(pcap_t *a, int b)
 {
-	g_assert(has_wpcap && pcap_set_timeout != NULL);
+	g_assert(has_wpcap && p_pcap_set_timeout != NULL);
 	return p_pcap_set_timeout(a, b);
 }
 int
 pcap_set_buffer_size(pcap_t *a, int b)
 {
-	g_assert(has_wpcap && pcap_set_timeout != NULL);
+	g_assert(has_wpcap && p_pcap_set_buffer_size != NULL);
 	return p_pcap_set_buffer_size(a, b);
 }
 
 int
 pcap_activate(pcap_t *a)
 {
-	g_assert(has_wpcap && pcap_activate != NULL);
+	g_assert(has_wpcap && p_pcap_activate != NULL);
 	return p_pcap_activate(a);
 
 }
@@ -527,8 +527,8 @@ pcap_statustostr(int a)
     static char ebuf[15 + 10 + 1];
 
     g_assert(has_wpcap);
-    if (pcap_statustostr != NULL) {
-        return pcap_statustostr(a);
+    if (p_pcap_statustostr != NULL) {
+        return p_pcap_statustostr(a);
     }
 
     /* XXX copy routine from pcap.c ??? */
@@ -975,7 +975,8 @@ cant_get_if_list_error_message(const char *err_str)
 }
 
 if_capabilities_t *
-get_if_capabilities_local(interface_options *interface_opts, char **err_str)
+get_if_capabilities_local(interface_options *interface_opts,
+    cap_device_open_err *err, char **err_str)
 {
 	/*
 	 * We're not getting capaibilities for a remote device; use
@@ -984,14 +985,15 @@ get_if_capabilities_local(interface_options *interface_opts, char **err_str)
 	 */
 #ifdef HAVE_PCAP_CREATE
 	if (p_pcap_create != NULL)
-		return get_if_capabilities_pcap_create(interface_opts, err_str);
+		return get_if_capabilities_pcap_create(interface_opts, err, err_str);
 #endif
-	return get_if_capabilities_pcap_open_live(interface_opts, err_str);
+	return get_if_capabilities_pcap_open_live(interface_opts, err, err_str);
 }
 
 pcap_t *
 open_capture_device_local(capture_options *capture_opts,
     interface_options *interface_opts, int timeout,
+    cap_device_open_err *open_err,
     char (*open_err_str)[PCAP_ERRBUF_SIZE])
 {
 	/*
@@ -1002,10 +1004,10 @@ open_capture_device_local(capture_options *capture_opts,
 #ifdef HAVE_PCAP_CREATE
 	if (p_pcap_create != NULL)
 		return open_capture_device_pcap_create(capture_opts,
-		    interface_opts, timeout, open_err_str);
+		    interface_opts, timeout, open_err, open_err_str);
 #endif
 	return open_capture_device_pcap_open_live(interface_opts, timeout,
-	    open_err_str);
+	    open_err, open_err_str);
 }
 
 /*
