@@ -162,12 +162,21 @@ struct ieee80211_radiotap_header {
  *	Contains the AMPDU information for the subframe.
  *
  * IEEE80211_RADIOTAP_HE		u16, u16, u16, u16, u16, u16 unitless
- *
  *	Contains some information for HE frames.
- *	NOTE: this is a suggested field, not an assigned field, so
- *	the type value and format of this field is subject to change,
- *	although an experimental Linux patch uses bit 23 and the
- *	current format on radiotap.org.
+ *
+ * IEEE80211_RADIOTAP_HE_MU		U16, U16, u8[4] unitless
+ *
+ * IEEE80211_RADIOTAP_HE_MU_USER
+ *
+ * IEEE80211_RADIOTAP_0_LENGTH_PSDU
+ *
+ * IEEE80211_RADIOTAP_L_SIG
+ *
+ * IEEE80211_RADIOTAP_TLVS_PRESENT
+ *
+ * THERE ARE NO MORE BITS FREE! If you need a new radiotap header you must
+ * ask for a TLV value. See www.radiotap.org.
+ *
  */
 enum ieee80211_radiotap_type {
 	IEEE80211_RADIOTAP_TSFT = 0,
@@ -194,6 +203,11 @@ enum ieee80211_radiotap_type {
 	IEEE80211_RADIOTAP_VHT = 21,
 	IEEE80211_RADIOTAP_TIMESTAMP = 22,
 	IEEE80211_RADIOTAP_HE = 23,
+	IEEE80211_RADIOTAP_HE_MU = 24,
+	IEEE80211_RAFIOTAP_HE_MU_USER = 25,
+	IEEE80211_RADIOTAP_0_LENGTH_PSDU = 26,
+	IEEE80211_RADIOTAP_L_SIG = 27,
+	IEEE80211_RADIOTAP_TLVS_PRESENT = 28,
 
 	/* valid in every it_present bitmap, even vendor namespaces */
 	IEEE80211_RADIOTAP_RADIOTAP_NAMESPACE = 29,
@@ -203,7 +217,7 @@ enum ieee80211_radiotap_type {
 
 /* not (yet) defined Radiotap present flag */
 /* Bit 24 to 28 are not defined (in binary : 0001 1111 0000 0000 0000 0000 0000 0000 */
-#define IEEE80211_RADIOTAP_NOTDEFINED 0x1F000000
+#define IEEE80211_RADIOTAP_NOTDEFINED 0x1E000000
 
 /* Channel flags. */
 /* 0x00000001 through 0x00000008 undefined (reserved?) */
@@ -431,6 +445,28 @@ enum ieee80211_radiotap_type {
 #define IEEE80211_RADIOTAP_HE_TXOP_VALUE_MASK			0x7F00
 #define IEEE80211_RADIOTAP_HE_MIDAMBLE_PERIODICITY		0x8000
 
+/* For IEEE80211_RADIOTAP_HE_MU */
+#define IEEE80211_RADIOTAP_HE_MU_SIG_B_MCS_MASK				0x000F
+#define IEEE80211_RADIOTAP_HE_MU_SIG_B_MCS_KNOWN			0x0010
+#define IEEE80211_RADIOTAP_HE_MU_SIG_B_DCM				0x0020
+#define IEEE80211_RADIOTAP_HE_MU_SIG_B_DCM_KNOWN			0x0040
+#define IEEE80211_RADIOTAP_HE_MU_RESERVED_F1_B7				0x0080
+#define IEEE80211_RADIOTAP_HE_MU_RU_0_KNOWN				0x0100
+#define IEEE80211_RADIOTAP_HE_MU_RU_1_KNOWN				0x0200
+#define IEEE80211_RADIOTAP_HE_MU_RU_2_KNOWN				0x0400
+#define IEEE80211_RADIOTAP_HE_MU_RU_3_KNOWN				0x0800
+#define IEEE80211_RADIOTAP_HE_MU_CENTER_26_TONE_RU_BIT_KNOWN		0x1000
+#define IEEE80211_RADIOTAP_HE_MU_CENTER_26_TONE_RU_VALUE		0x2000
+#define IEEE80211_RADIOTAP_HE_MU_SIG_B_COMPRESSION_KNOWN		0x4000
+#define IEEE80211_RADIOTAP_HE_MU_SYMBOL_CNT_OR_USER_CNT_KNOWN		0x8000
+#define IEEE80211_RADIOTAP_HE_MU_BW_FROM_BW_IN_SIG_A_MASK		0x0003
+#define IEEE80211_RADIOTAP_HE_MU_BW_FROM_BW_IN_SIG_A_KNOWN		0x0004
+#define IEEE80211_RADIOTAP_HE_MU_SIG_B_COMPRESSION_FROM_SIG_A		0x0008
+#define IEEE80211_RADIOTAP_HE_MU_SYMBOL_CNT_OR_USER_CNT			0x00F0
+#define IEEE80211_RADIOTAP_HE_MU_PREAMBLE_PUNCTURING_MASK		0x0300
+#define IEEE80211_RADIOTAP_HE_MU_PREAMBLE_PUNCTURING_KNOWN		0x0400
+#define IEEE80211_RADIOTAP_HE_MU_RESERVED_F2_B11_B15			0xF800
+
 #define IEEE80211_RADIOTAP_TS_FLG_32BIT		0x01
 #define IEEE80211_RADIOTAP_TS_FLG_ACCURACY	0x02
 
