@@ -6,19 +6,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -49,7 +37,7 @@ static char *skipWhiteSpace(char *source, int *accumulated_offset)
     int offset = 0;
 
     /* Skip any leading whitespace */
-    while (source[offset] != '\0' && source[offset] == ' ') {
+    while ((source[offset] == ' ') || (source[offset] == '\t')) {
         offset++;
     }
 
@@ -480,8 +468,15 @@ char *expand_reference(SnortConfig_t *snort_config, char *reference)
         prefix_replacement = (char*)g_hash_table_lookup(snort_config->references_prefixes, prefix);
 
         /* Append prefix and remainder, and return!!!! */
-        g_snprintf(expanded_reference, 512, "%s%s", prefix_replacement, reference+length+1);
-        return expanded_reference;
+        if (prefix_replacement) {
+            g_snprintf(expanded_reference, 512, "%s%s", prefix_replacement, reference+length+1);
+            return expanded_reference;
+        }
+        else {
+            /* Just return the original reference */
+            return reference;
+        }
+
     }
     return "ERROR: Reference didn't contain prefix and ','!";
 }
