@@ -575,6 +575,7 @@ proto_tree_write_node_pdml(proto_node *node, gpointer data)
                         case FT_INT32:
                             fprintf(pdata->fh, "%X", (guint) fvalue_get_sinteger(&fi->value));
                             break;
+                        case FT_CHAR:
                         case FT_UINT8:
                         case FT_UINT16:
                         case FT_UINT24:
@@ -960,6 +961,7 @@ write_json_proto_node_hex_dump(proto_node *node, write_json_data *data)
             case FT_INT32:
                 fprintf(data->fh, "%X", (guint) fvalue_get_sinteger(&fi->value));
                 break;
+            case FT_CHAR:
             case FT_UINT8:
             case FT_UINT16:
             case FT_UINT24:
@@ -1138,8 +1140,12 @@ static gboolean
 ek_check_protocolfilter(gchar **protocolfilter, const char *str)
 {
     gchar *str_escaped = NULL;
+    gboolean check;
     int i;
 
+    if (check_protocolfilter(protocolfilter, str))
+        return TRUE;
+
     /* to to thread the '.' and '_' equally. The '.' is replace by print_escaped_ek for '_' */
     if (str != NULL && strlen(str) > 0) {
         str_escaped = g_strdup(str);
@@ -1153,8 +1159,9 @@ ek_check_protocolfilter(gchar **protocolfilter, const char *str)
         }
     }
 
-    return check_protocolfilter(protocolfilter, str)
-           || check_protocolfilter(protocolfilter, str_escaped);
+    check = check_protocolfilter(protocolfilter, str_escaped);
+    g_free(str_escaped);
+    return check;
 }
 
 /**
@@ -1270,6 +1277,7 @@ ek_write_hex(field_info *fi, write_json_data *pdata)
             case FT_INT32:
                 fprintf(pdata->fh, "%X", (guint) fvalue_get_sinteger(&fi->value));
                 break;
+            case FT_CHAR:
             case FT_UINT8:
             case FT_UINT16:
             case FT_UINT24:
@@ -1465,7 +1473,8 @@ proto_tree_write_node_ek(proto_node *node, write_json_data *pdata)
     g_hash_table_destroy(attr_table);
 
     // Print attributes
-    GSList *current_attr = g_slist_reverse(attr_list);
+    attr_list = g_slist_reverse(attr_list);
+    GSList *current_attr = attr_list;
     while (current_attr != NULL) {
         GSList *attr_instances = (GSList *) current_attr->data;
 
@@ -1763,7 +1772,6 @@ get_field_data(GSList *src_list, field_info *fi)
             return tvb_get_ptr(src_tvb, fi->start, length);
         }
     }
-    g_assert_not_reached();
     return NULL;  /* not found */
 }
 
