@@ -431,8 +431,6 @@ static void analyze_gop(mate_config* mc, mate_gop* gop) {
 				/** Can't get here because of "breaks" above */
 				g_assert_not_reached();
 			}
-
-			dbg_print (dbg_gog,1,dbg_facility,"analyze_gop: no gogkey_match: %s",key);
 		} /* while */
 
 		g_free(key);
@@ -666,7 +664,7 @@ static void get_pdu_fields(gpointer k, gpointer v, gpointer p) {
 			start = fi->start;
 			end = fi->start + fi->length;
 
-			dbg_print(dbg_pdu,5,dbg_facility,"get_pdu_fields: found field %i-%i",start,end);
+			dbg_print(dbg_pdu,5,dbg_facility,"get_pdu_fields: found field %s, %i-%i, length %i", fi->hfinfo->abbrev, start, end, fi->length);
 
 			for (j = 0; j < data->ranges->len; j++) {
 
@@ -691,6 +689,11 @@ static void get_pdu_fields(gpointer k, gpointer v, gpointer p) {
 	}
 }
 
+static void ptr_array_free(gpointer data, gpointer user_data _U_)
+{
+	g_free(data);
+}
+
 static mate_pdu* new_pdu(mate_cfg_pdu* cfg, guint32 framenum, field_info* proto, proto_tree* tree) {
 	mate_pdu* pdu = (mate_pdu*)g_slice_new(mate_max_size);
 	field_info* cfi;
@@ -813,7 +816,7 @@ static mate_pdu* new_pdu(mate_cfg_pdu* cfg, guint32 framenum, field_info* proto,
 
 	apply_transforms(pdu->cfg->transforms,pdu->avpl);
 
-	g_ptr_array_foreach(data.ranges, (GFunc)g_free, NULL);
+	g_ptr_array_foreach(data.ranges, ptr_array_free, NULL);
 	g_ptr_array_free(data.ranges,TRUE);
 
 	return pdu;
