@@ -10,7 +10,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 
@@ -27,9 +27,7 @@ typedef enum {
 
 int parse_token(token_t token, char *str);
 
-int text2pcap_lex(void);
-
-int text2pcap_lex_destroy(void);
+int text2pcap_scan(void);
 
 #endif
 
