@@ -23,6 +23,7 @@
 #include <wsutil/frequency-utils.h>
 
 #include <QProcess>
+#include <QAbstractItemView>
 
 // To do:
 // - Disable or hide invalid channel types.
@@ -46,7 +47,7 @@ WirelessFrame::WirelessFrame(QWidget *parent) :
 
     ui->helperToolButton->hide();
 
-    if (ws80211_init() == 0) {
+    if (ws80211_init() == WS80211_INIT_OK) {
         ui->stackedWidget->setEnabled(true);
         ui->stackedWidget->setCurrentWidget(ui->interfacePage);
 
